package be.iminds.ilabt.jfed.restapi.dao;

import be.iminds.ilabt.jfed.fedmon.util.Clock;
import be.iminds.ilabt.jfed.restapi.JFedRestApiApplication;
import be.iminds.ilabt.jfed.restapi.JFedRestApiConfiguration;
import be.iminds.ilabt.jfed.restapi.TestDB;
import be.iminds.ilabt.jfed.restapi.json.Experiment;
import be.iminds.ilabt.jfed.restapi.json.ExperimentBuilder;
import be.iminds.ilabt.jfed.restapi.json.ExperimentTestCollection;
import be.iminds.ilabt.util.jsonld.test.CommonTest;
import io.dropwizard.jdbi.DBIFactory;
import io.dropwizard.setup.Environment;
import io.dropwizard.testing.junit.DropwizardAppRule;
import org.junit.*;
import org.skife.jdbi.v2.DBI;

import java.sql.SQLException;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.Matchers.is;
import static org.hamcrest.Matchers.notNullValue;

public class ExperimentDaoTest {
    @ClassRule
    public static final DropwizardAppRule<JFedRestApiConfiguration> RULE =
            new DropwizardAppRule<JFedRestApiConfiguration>(JFedRestApiApplication.class, "../jfed-rest-api-TESTING.yml");

    private static TestDB testDB;
    private static DBI jdbi;
    private static ExperimentDao experimentDao;
    //lock to make sure that nothing tries to parallellise these tests
    private static Lock lock = new ReentrantLock();
    @BeforeClass
    public static void setUp() throws Exception {
        Clock.set(new Clock.DefaultTestTimeProvider());

        testDB = new TestDB(RULE);
        testDB.dropAllTables();
        testDB.syncTableStructure();
        testDB.refillDB();

        JFedRestApiConfiguration configuration = RULE.getConfiguration();
        Environment environment = RULE.getEnvironment();
        final DBIFactory factory = new DBIFactory();
        jdbi = factory.build(environment, configuration.getDataSourceFactory(), "postgresql");
        experimentDao = jdbi.onDemand(ExperimentDao.class);
        assert experimentDao != null;
    }
    @AfterClass
    public static void tearDown() throws Exception {
        assert testDB != null;
        jdbi.close(experimentDao);
        jdbi = null;
        experimentDao = null;
        testDB = null;
        System.gc();
        Clock.reset();
    }

    private ExperimentTestCollection experimentTestCollection;

    @Before
    public void beforeEachMethod() throws Exception {
        lock.lock();

        assert testDB != null;
        testDB.refillDB();

        assert experimentDao != null;

        experimentTestCollection = new ExperimentTestCollection();
    }


    @After
    public void afterEachMethod() throws SQLException {
        lock.unlock();
    }

    @Test
    public void testGetById0() throws Exception {
        assert experimentDao != null;
        Experiment expected = experimentTestCollection.getByIndex(0);
        Experiment actual= experimentDao.findById(expected.getId());
        System.out.println("findById res = "+actual);
        assertThat(actual, is(notNullValue()));
        experimentTestCollection.assertSame(actual, expected,
                CommonTest.ComparePrecision.IGNORE_URI,
                CommonTest.ComparePrecision.IGNORE_URI);
    }

    @Test
    public void testInsertAndFind0() throws Exception {
        assert experimentDao != null;
        ExperimentBuilder expectedNew = experimentTestCollection.getBuilderByIndex(0);
        expectedNew.setRequestRspec("DUMMY REQUEST RSPEC");
        assertThat(expectedNew.create().getStatusAsString(), is(equalTo("STARTING")));

        Long newId = experimentDao.insert(expectedNew.create());

        expectedNew.setId(newId);
        Experiment expected = expectedNew.create();

        Experiment actual= experimentDao.findById(expected.getId());
        System.out.println("findById res = "+actual);
        assertThat(actual, is(notNullValue()));
        experimentTestCollection.assertSame(actual, expected,
                        CommonTest.ComparePrecision.IGNORE_URI,
                        CommonTest.ComparePrecision.IGNORE_URI);
    }

    @Test
    public void testUpdate() throws Exception {
        assert experimentDao != null;

        ExperimentBuilder original = experimentTestCollection.getBuilderByIndex(0);
        ExperimentBuilder expectedUpdated = experimentTestCollection.getBuilderByIndex(1);
        expectedUpdated.setId(original.getId());

        experimentDao.update(expectedUpdated.create());

        Experiment actual= experimentDao.findById(original.getId());
        System.out.println("testUpdate findById res = "+actual);
        assertThat(actual, is(notNullValue()));
        experimentTestCollection.assertSame(actual, expectedUpdated.create(),
                CommonTest.ComparePrecision.IGNORE_URI,
                CommonTest.ComparePrecision.IGNORE_URI);
    }

}
